﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Data;
#if INCLUDE_WINFORMS
using C1.Win.FlexGrid;
#endif

namespace WPFGridsBenchmark
{
    public enum DataSourceType
    {
        ListCollectionView,
        DataView
    }

    #region ** Core
    public abstract class TestGrid : UserControl
    {
        public static Dictionary<string, Type> Grids;
        public static List<Type> GridsForAutoSizeTesting;
        static TestGrid()
        {
            Grids = new Dictionary<string, Type>();
            GridsForAutoSizeTesting = new List<Type>();
            Grids.Add("Microsoft's DataGrid for WPF", typeof(MSDataGrid));
            GridsForAutoSizeTesting.Add(typeof(MSDataGrid));
            Grids.Add("FlexGrid for WPF", typeof(C1FlexGrid));

#if INCLUDE_WINFORMS
            Grids.Add("FlexGrid for WinForms", typeof(WinFormsFlexGrid));
#endif

#if INCLUDE_COMPETITORS
            Grids.Add("Syncfusion SfDataGrid", typeof(SFDataGrid));
            GridsForAutoSizeTesting.Add(typeof(SFDataGrid));
            Grids.Add("DevExpress GridControl", typeof(DXGridControl));
            Grids.Add("Infragistics XamDataGrid", typeof(InfragisticsDataGrid));
            GridsForAutoSizeTesting.Add(typeof(InfragisticsDataGrid));
            Grids.Add("Telerik TelerikRadGridView", typeof(TelerikRadGridView));
            GridsForAutoSizeTesting.Add(typeof(TelerikRadGridView));
#endif
        }

        protected void SetGrid(object grid)
        {
            this.Content = grid;
        }
        public virtual DataSourceType AcceptableDataSource
        {
            get
            {
                return DataSourceType.ListCollectionView;
            }
        }
        public virtual void Load(IEnumerable data)
        {
            ItemsSource = data;
        }
        public virtual void Dispose()
        {
            var cv = ItemsSource as ListCollectionView;
            ItemsSource = null;
            this.Content = null;
            if (cv != null)
            {
                cv.Filter = null;
            }
        }
        public virtual void Sort(bool ascending)
        {
            ICollectionView dataView = ItemsSource as ICollectionView;
            using (dataView.DeferRefresh())
            {
                dataView.SortDescriptions.Clear();
                dataView.SortDescriptions.Add(new SortDescription("ID", ascending ? ListSortDirection.Ascending : ListSortDirection.Descending));
            }
        }

        public virtual string GetGridInfo()
        {
            if (this.Content != null)
            {
                System.Reflection.AssemblyName asName = this.Content.GetType().Assembly.GetName();
                return asName.Name + "." + asName.Version;
            }
            return String.Empty;
        }
        public abstract void ScrollInView(int rowNumber);

        public virtual async Task Filter(bool isActive)
        {
            var cv = ItemsSource as ListCollectionView;
            if (cv != null)
            {
                if (isActive)
                    cv.Filter = IsActiveFilter;
                else
                    cv.Filter = IsInactiveFilter;
            }
        }

        bool IsActiveFilter(object customer)
        {
            return ((Customer)customer).Active;
        }

        bool IsInactiveFilter(object customer)
        {
            return !((Customer)customer).Active;
        }
        public virtual async Task Group()
        {
            ICollectionView cv = ItemsSource as ICollectionView;
            using (cv.DeferRefresh())
            {
                cv.GroupDescriptions.Clear();
                cv.GroupDescriptions.Add(new PropertyGroupDescription("CountryID"));
            }
        }

        public abstract IEnumerable ItemsSource { get; set; }
        public virtual void BeginUpdate()
        {
        }
        public virtual void EndUpdate()
        {
        }
    }

    public class MSDataGrid : TestGrid
    {
        DataGrid _grid = null;
        public MSDataGrid(bool fixedColumnWidth)
        {
            _grid = new DataGrid();
            // default column sizing is SizeToHeader, finally sets column width by content
            if (fixedColumnWidth)
            {
                _grid.ColumnWidth = new DataGridLength(150);
            }
            SetGrid(_grid);
        }

        public override IEnumerable ItemsSource
        {
            get => _grid.ItemsSource;
            set => _grid.ItemsSource = value;
        }

        public override void ScrollInView(int rowNumber)
        {
            object item = ((ListCollectionView)_grid.ItemsSource).GetItemAt(rowNumber);
            _grid.ScrollIntoView(item);
        }
    }

    public class C1FlexGrid : TestGrid
    {
        C1.WPF.Grid.FlexGrid _grid = null;

        public C1FlexGrid(bool fixedColumnWidth)
        {
            _grid = new C1.WPF.Grid.FlexGrid();
            // uses some predefined column width depending on underlying data
            // auto-width is only possible with method call
            _grid.NewRowPosition = C1.WPF.Grid.GridNewRowPosition.Bottom;
            _grid.ColumnOptionsMenuVisibility = C1.WPF.Grid.GridColumnOptionsMenuVisibility.Collapsed;
            _grid.IsAnimated = false; // disable animations
            SetGrid(_grid);
        }

        public override IEnumerable ItemsSource
        {
            get => _grid.ItemsSource;
            set => _grid.ItemsSource = value;
        }

        public override void ScrollInView(int rowNumber)
        {
            _grid.ScrollIntoView(rowNumber, 0, disableAnimation: true); // disable animation
        }

    }
    #endregion

    #region ** Competitors
#if INCLUDE_COMPETITORS
    // Telerik - did not find any NuGet servers, you can download zip files from Telerik's site to use local NuGet.
    public class TelerikRadGridView : TestGrid
    {
        Telerik.Windows.Controls.RadGridView _grid = null;

        public TelerikRadGridView(bool fixedColumnWidth)
        {
            _grid = new Telerik.Windows.Controls.RadGridView();
            _grid.ShowSearchPanel = false;
            _grid.ShowGroupPanel = false;
            _grid.AutoExpandGroups = true; // to make it similar with other grids
            _grid.IsFilteringAllowed = false;
            // default column sizing is SizeToHeader, finally sizes columns to show all content
            if (fixedColumnWidth)
            {
                _grid.ColumnWidth = new Telerik.Windows.Controls.GridViewLength(150);
            }
            _grid.NewRowPosition = Telerik.Windows.Controls.GridView.GridViewNewRowPosition.Bottom;
            SetGrid(_grid);
        }

        public override IEnumerable ItemsSource
        {
            get => (IEnumerable)_grid.ItemsSource;
            set => _grid.ItemsSource = value;
        }

        public override void ScrollInView(int rowNumber)
        {
            _grid.ScrollIndexIntoView(rowNumber);
        }
    }

    // only have library with .Net Core 3 target 
    // Have own nuget feed: https://nuget.devexpress.com/
    public class DXGridControl : TestGrid
    {
        DevExpress.Xpf.Grid.GridControl _grid = null;
        public DXGridControl(bool fixedColumnWidth)
        {
            _grid = new DevExpress.Xpf.Grid.GridControl();
            _grid.AutoGenerateColumns = DevExpress.Xpf.Grid.AutoGenerateColumnsMode.AddNew;
            var tableView = new DevExpress.Xpf.Grid.TableView();
            _grid.View = tableView;
            tableView.AllowScrollAnimation = false; // disable animations
            tableView.UseAnimationWhenExpanding = false; // disable animations
            tableView.ShowGroupPanel = false;
            tableView.AllowColumnFiltering = false;
            // by default columns have fixed size
            // autowidth is only possible by method calls
            tableView.NewItemRowPosition = DevExpress.Xpf.Grid.NewItemRowPosition.Bottom;
            SetGrid(_grid);
        }

        public override IEnumerable ItemsSource
        {
            get => (IEnumerable)_grid.ItemsSource;
            set => _grid.ItemsSource = value;
        }

        public override void ScrollInView(int rowNumber)
        {
            _grid.View.ScrollIntoView(rowNumber);
        }
        public override void BeginUpdate()
        {
            _grid.BeginDataUpdate();
        }

        public override void EndUpdate()
        {
            _grid.EndDataUpdate();
        }
    }

    // only have library with .Net 5 target
    public class SFDataGrid : TestGrid
    {
        Syncfusion.UI.Xaml.Grid.SfDataGrid _grid = null;

        public SFDataGrid(bool fixedColumnWidth)
        {
            // trial key
            Syncfusion.Licensing.SyncfusionLicenseProvider.RegisterLicense("NTQxMzA4QDMxMzkyZTMzMmUzMGNEbW1hWDFsTVU2T1ZkZklrc2Z6VXVva08xZHYycFRHTXdVOEd6QUkzZTA9");
            // NTQxMzA4QDMxMzkyZTMzMmUzMGNEbW1hWDFsTVU2T1ZkZklrc2Z6VXVva08xZHYycFRHTXdVOEd6QUkzZTA9
            _grid = new Syncfusion.UI.Xaml.Grid.SfDataGrid();
            _grid.AllowEditing = true;
            _grid.AllowGrouping = false;
            _grid.AutoExpandGroups = true; // it is similar to all other grids in these tests
                                           // Uses fixed column width by default. No autostar setting
            if (!fixedColumnWidth) // it's extremely slow on large data, exclude it from testing https://www.syncfusion.com/forums/120859/sfdatagrid-trouble-rendering
            {
                _grid.ColumnSizer = Syncfusion.UI.Xaml.Grid.GridLengthUnitType.AutoWithLastColumnFill;
            }
            _grid.AddNewRowPosition = Syncfusion.UI.Xaml.Grid.AddNewRowPosition.Bottom;
            SetGrid(_grid);
        }


        public override IEnumerable ItemsSource
        {
            get => (IEnumerable)_grid.ItemsSource;
            set => _grid.ItemsSource = value;
        }

        public override void Sort(bool ascending)
        {
            _grid.SortColumnDescriptions.Clear();
            Syncfusion.UI.Xaml.Grid.SortColumnDescription sort = new Syncfusion.UI.Xaml.Grid.SortColumnDescription();
            sort.ColumnName = "ID";
            sort.SortDirection = ascending ? ListSortDirection.Ascending : ListSortDirection.Descending;
            _grid.SortColumnDescriptions.Add(sort);
        }

        public override async Task Group()
        {
            _grid.GroupColumnDescriptions.Clear();
            _grid.GroupColumnDescriptions.Add(new Syncfusion.UI.Xaml.Grid.GroupColumnDescription() { ColumnName = "CountryID" });
        }

        public override void ScrollInView(int rowNumber)
        {
            _grid.ScrollInView(new Syncfusion.UI.Xaml.ScrollAxis.RowColumnIndex() { RowIndex = rowNumber, ColumnIndex = 0 });
        }
        public override async Task Filter(bool isActive)
        {
            if (isActive)
                _grid.View.Filter = IsActiveFilter;
            else
                _grid.View.Filter = IsInactiveFilter;
            _grid.View.RefreshFilter();
        }

        bool IsActiveFilter(object customer)
        {
            return ((Customer)customer).Active;
        }

        bool IsInactiveFilter(object customer)
        {
            return !((Customer)customer).Active;
        }
    }

    // only have library with .Net 5 target
    public class InfragisticsDataGrid : TestGrid
    {
        Infragistics.Windows.DataPresenter.XamDataGrid _grid = null;
        public InfragisticsDataGrid(bool fixedColumnWidth)
        {
            _grid = new Infragistics.Windows.DataPresenter.XamDataGrid();
            // default column sizing uses fixed column width. Don't see good way to change it
            _grid.GroupByAreaLocation = Infragistics.Windows.DataPresenter.GroupByAreaLocation.None;
            _grid.FieldLayoutSettings.AllowAddNew = true;
            // by default columns have fixed size
            if (!fixedColumnWidth)
            {
                _grid.FieldSettings.Width = Infragistics.Windows.DataPresenter.FieldLength.InitialAuto;
                _grid.FieldSettings.AutoSizeScope = Infragistics.Windows.DataPresenter.FieldAutoSizeScope.RecordsInView;
                _grid.FieldSettings.AutoSizeOptions = Infragistics.Windows.DataPresenter.FieldAutoSizeOptions.All;
            }
            _grid.FieldLayoutSettings.AddNewRecordLocation = Infragistics.Windows.DataPresenter.AddNewRecordLocation.OnBottom;
            SetGrid(_grid);
        }

        public override IEnumerable ItemsSource
        {
            get => _grid.DataSource;
            set => _grid.DataSource = value;
        }

        public override void ScrollInView(int rowNumber)
        {
            object item = ((ListCollectionView)_grid.DataSource).GetItemAt(rowNumber);
            _grid.BringDataItemIntoView(item, false);
        }
    }

#endif
    #endregion

    #region ** WinForms
#if INCLUDE_WINFORMS
    public class WinFormsFlexGrid : TestGrid
    {
        C1.Win.FlexGrid.C1FlexGrid _grid = null;
        System.Windows.Forms.Integration.WindowsFormsHost _host = null;
        public WinFormsFlexGrid(bool fixedColumnWidth)
        {
            _grid = new C1.Win.FlexGrid.C1FlexGrid();
            _host = new System.Windows.Forms.Integration.WindowsFormsHost();
            _host.Background = new System.Windows.Media.SolidColorBrush(System.Windows.Media.Colors.White); // to keep performance high, always use non-transparent background
            _host.Child = _grid;
            // uses some predefined column width depending on underlying data
            // auto-width is only possible with method call
            _grid.AllowAddNew = true;
            SetGrid(_host);
        }

        public override void Dispose()
        {
            var cv = _grid.DataSource as DataView;
            _grid.DataSource = null;
            cv.RowFilter = null;
            _host.Dispose();
            _host = null;
            base.Dispose();
        }

        public override DataSourceType AcceptableDataSource
        {
            get
            {
                return DataSourceType.DataView;
            }
        }

        public override IEnumerable ItemsSource
        {
            get => _grid.DataSource as IEnumerable;
            set => _grid.DataSource = value;
        }

        public override void Sort(bool ascending)
        {
            _grid.Sort(ascending ? C1.Win.FlexGrid.SortFlags.Ascending : C1.Win.FlexGrid.SortFlags.Descending, 1);
        }
        public override void ScrollInView(int rowNumber)
        {
            _grid.ShowCell(rowNumber, _grid.Cols.Fixed);
        }
        public async override Task Filter(bool isActive)
        {
            var cv = _grid.DataSource as DataView;
            cv.RowFilter = ("Active = " + isActive.ToString(System.Globalization.CultureInfo.InvariantCulture));
        }

        public override async Task Group()
        {
            _grid.BeginUpdate();
            _grid.GroupDescriptions = null;
            _grid.GroupDescriptions = new List<C1.Win.FlexGrid.GroupDescription>() { new C1.Win.FlexGrid.GroupDescription("CountryID") };
            _grid.EndUpdate();
        }

        public override string GetGridInfo()
        {
            if (this._grid != null)
            {
                System.Reflection.AssemblyName asName = _grid.GetType().Assembly.GetName();
                return asName.Name + "." + asName.Version;
            }
            return String.Empty;
        }
    }
#endif
    #endregion

}
